@@ -25,6 +25,7 @@ const GObject = imports.gi.GObject;
 const Geocode = imports.gi.GeocodeGlib;
 const GtkChamplain = imports.gi.GtkChamplain;
 const Lang = imports.lang;
+const Mainloop = imports.mainloop;
 
 const Application = imports.application;
 const ContactPlace = imports.contactPlace;
@@ -47,7 +48,7 @@ const MapType = {
     CYCLING: Champlain.MAP_SOURCE_OSM_CYCLE_MAP,
     TRANSIT: Champlain.MAP_SOURCE_OSM_TRANSPORT_MAP
 };
-
+const _LOCATION_STORE_TIMEOUT = 500;
 const MapMinZoom = 2;
 
 const MapView = new Lang.Class({
@@ -91,7 +92,7 @@ const MapView = new Lang.Class({
                                     this._updateUserLocation.bind(this));
         Application.geoclue.connect('notify::state',
                                     this._updateUserLocation.bind(this));
-
+        this._storeId = 0;
         this._connectRouteSignals();
     },
 
@@ -103,8 +104,8 @@ const MapView = new Lang.Class({
         view.reactive = true;
         view.kinetic_mode = true;
 
+        view.connect('notify::realized', this._goToStoredLocation.bind(this));
         view.connect('notify::latitude', this._onViewMoved.bind(this));
-        view.connect('notify::longitude', this._onViewMoved.bind(this));
         // switching map type will set view min-zoom-level from map source
         view.connect('notify::min-zoom-level', (function() {
             if (view.min_zoom_level < MapMinZoom) {
@@ -184,10 +185,7 @@ const MapView = new Lang.Class({
         }
     },
 
-    _openGeoJSONInternal: function(file) {
-        if (!this.view.realized)
-            return;
-
+    openGeoJSON: function(file) {
         try {
             this._annotationMarkerLayer.remove_all();
             let geoJSONSource = new GeoJSONSource.GeoJSONSource({
@@ -211,12 +209,22 @@ const MapView = new Lang.Class({
         }
     },
 
-    openGeoJSON: function(file) {
-        if (this.view.realized) {
-            this._openGeoJSONInternal(file);
-        } else {
-            this.view.connect('notify::realized',
-                              this._openGeoJSONInternal.bind(this, file));
+    goToGeoURI: function(uri) {
+        try {
+            let location = new Location.Location({ heading: -1 });
+            location.set_from_uri(uri);
+
+            let place = new Place.Place({ location: location,
+                                          name: location.description,
+                                          store: false });
+            let marker = new PlaceMarker.PlaceMarker({ place: place,
+                                                       mapView: this });
+            this._placeLayer.add_marker(marker);
+            marker.goToAndSelect(true);
+        } catch(e) {
+            let msg = _("Failed to open GeoURI");
+            Application.notificationManager.showMessage(msg);
+            Utils.debug("failed to open GeoURI: %s".format(e.message));
         }
     },
 
@@ -261,7 +269,25 @@ const MapView = new Lang.Class({
         this.emit('user-location-changed');
     },
 
-    _gotoBBox: function(bbox) {
+    _storeLocation: function() {
+        let box = this.view.get_bounding_box();
+        let lastViewedLocation = [box.top, box.bottom, box.left, box.right];
+        Application.settings.set('last-viewed-location', lastViewedLocation);
+    },
+
+    _goToStoredLocation: function() {
+        if (!this.view.realized)
+            return;
+
+        let box = Application.settings.get('last-viewed-location');
+        let bounding_box = new Champlain.BoundingBox({ top: box[0],
+                                                       bottom: box[1],
+                                                       left: box[2],
+                                                       right: box[3] });
+        this._gotoBBox(bounding_box, true);
+    },
+
+    _gotoBBox: function(bbox, linear) {
         let [lat, lon] = bbox.get_center();
         let place = new Place.Place({
             location: new Location.Location({ latitude  : lat,
@@ -271,7 +297,7 @@ const MapView = new Lang.Class({
                                                     left   : bbox.left,
                                                     right  : bbox.right })
         });
-        new MapWalker.MapWalker(place, this).goTo(true);
+        new MapWalker.MapWalker(place, this).goTo(true, linear);
     },
 
     showTurnPoint: function(turnPoint) {
@@ -380,6 +406,13 @@ const MapView = new Lang.Class({
 
     _onViewMoved: function() {
         this.emit('view-moved');
+        if (this._storeId !== 0)
+            return;
+
+        this._storeId = Mainloop.timeout_add(_LOCATION_STORE_TIMEOUT,(function(){
+            this._storeId = 0;
+            this._storeLocation();
+        }).bind(this));
     },
 
     onSetMarkerSelected: function(selectedMarker) {
