@@ -17,6 +17,7 @@
  * Author: Jonas Danielson <jonas@threetimestwo.org>
  */
 
+const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
@@ -45,6 +46,9 @@ const SendToDialog = new Lang.Class({
                         'weatherLabel',
                         'clocksRow',
                         'clocksLabel',
+                        'browserRow',
+                        'browserLabel',
+                        'browserIcon',
                         'headerBar',
                         'cancelButton',
                         'chooseButton',
@@ -54,6 +58,9 @@ const SendToDialog = new Lang.Class({
         this._place = params.place;
         delete params.place;
 
+        this._mapView = params.mapView;
+        delete params.mapView;
+
         params.use_header_bar = true;
         this.parent(params);
 
@@ -63,8 +70,14 @@ const SendToDialog = new Lang.Class({
         this._cancelButton.connect('clicked',
                                    this.response.bind(this, Response.CANCEL));
 
-        this._chooseButton.connect('clicked',
-                                   this._onChooseButtonClicked.bind(this));
+        this._chooseButton.connect('clicked', (function() {
+            let row = this._list.get_selected_row();
+            this._activateRow(row);
+        }).bind(this));
+
+        this._list.connect('row-activated', (function(list, row) {
+            this._activateRow(row);
+        }).bind(this));
 
         this._list.set_header_func(function(row, before) {
             let horizontal = Gtk.Orientation.HORIZONTAL;
@@ -79,6 +92,7 @@ const SendToDialog = new Lang.Class({
     ensureApplications: function() {
         let weatherInfo = Gio.DesktopAppInfo.new(_WEATHER_APPID + '.desktop');
         let clocksInfo = Gio.DesktopAppInfo.new(_CLOCKS_APPID + '.desktop');
+        let browserInfo = Gio.AppInfo.get_default_for_uri_scheme('https');
         let appWeather = this._checkWeather(weatherInfo);
         let appClocks = this._checkClocks(clocksInfo);
 
@@ -92,23 +106,45 @@ const SendToDialog = new Lang.Class({
         else
             this._clocksLabel.label = clocksInfo.get_name();
 
-        return appWeather || appClocks;
+        if (!browserInfo) {
+            this._browserRow.hide();
+        } else {
+            this._browserLabel.label = browserInfo.get_name();
+            this._browserIcon.icon_name = browserInfo.get_icon().to_string();
+        }
+
+        return appWeather || appClocks || browserInfo;
     },
 
-    _onChooseButtonClicked: function() {
-        let rows = this._list.get_selected_rows();
-        if (rows.length === 0)
-            this.response(Response.CANCEL);
+    _getOSMURI: function() {
+        let view = this._mapView.view;
+        let place = this._place;
 
-        if (rows[0] === this._weatherRow || rows[0] === this._clocksRow) {
+        let base = 'https://openstreetmap.org';
+        if (this._place.osm_id && this._place.osm_type) {
+            return '%s/%s/%s'.format(base,
+                                     Utils.osmTypeToString(place.osm_type),
+                                     place.osm_id);
+        } else {
+            return '%s?lat=%f&lon=%f&zoom=%d'.format(base,
+                                                     place.location.latitude,
+                                                     place.location.longitude,
+                                                     view.zoom_level);
+        }
+    },
+
+    _activateRow: function(row) {
+        let timestamp = Gtk.get_current_event_time();
+
+        if (row === this._weatherRow || row === this._clocksRow) {
             let location = this._place.location;
             let city = GWeather.Location.new_detached(this._place.name,
                                                       null,
                                                       location.latitude,
                                                       location.longitude);
-            let appId;
             let action;
-            if (rows[0] === this._weatherRow) {
+            let appId;
+            if (row === this._weatherRow) {
                 action = 'show-location';
                 appId = _WEATHER_APPID;
             } else {
@@ -119,10 +155,23 @@ const SendToDialog = new Lang.Class({
             Utils.activateAction(appId,
                                  action,
                                  new GLib.Variant('v', city.serialize()),
-                                 Gtk.get_current_event_time());
+                                 timestamp);
+        } else if (row === this._browserRow) {
+            try {
+                let display = Gdk.Display.get_default();
+                let ctx = Gdk.Display.get_default().get_app_launch_context();
+                let screen = display.get_default_screen();
 
-            this.response(Response.SUCCESS);
+                ctx.set_timestamp(timestamp);
+                ctx.set_screen(screen);
+                Gio.app_info_launch_default_for_uri(this._getOSMURI(), ctx);
+            } catch(e) {
+                let msg = _("Failed to open URI");
+                Application.notificationManager.showMessage(msg);
+                Utils.debug('failed to open URI: %s'.format(e.message));
+            }
         }
+        this.response(Response.SUCCESS);
     },
 
     _checkWeather: function(appInfo) {
